<?php
/**
 * @file
 * This include processes Custom URLs for use by emaudio.module.
 */

/**
 * Implementation of hook emaudio_PROVIDER_info().
 *
 * This returns information relevant to a specific 3rd party audio provider.
 *
 * @return
 *   An array of strings requested by various admin and other forms.
 *   'name' => The translated name of the provider.
 *   'url' => The url to the main page for the provider.
 *   'settings_description' => A description of the provider that will be posted in the admin settings form.
 *   'supported_features' => An array of rows describing the state of certain supported features by the provider.
 *      These will be rendered in a table, with the columns being 'Feature', 'Supported', 'Notes'.
 */
function emaudio_customurl_info() {
  $features = array(
    array(t('Autoplay'), t('Yes'), ''),
    array(t('RSS Attachment'), t('No'), ''),
    array(t('Thumbnails'), t('No'), t('')),
  );
  return array(
    'provider' => 'customurl',
    'name' => t('Custom URL'),
    'url' => 'http://drupal.org/project/media_customurl',
    'settings_description' => t('These settings specifically affect audio displayed by custom URLs.'),
    'supported_features' => $features,
  );
}

/**
 * Implementation of hook emaudio_PROVIDER_settings().
 *
 * This should return a subform to be added to the emaudio_settings() admin settings page.
 * Note that a form field will already be provided, at $form['odeo'] (such as $form['podomatic']).
 * So if you want specific provider settings within that field, you can add the elements to that form field.
 */
function emaudio_customurl_settings() {
  $form = array();
  $options = emaudio_customurl_default_types();
  $form['emaudio_customurl_supported_types'] = array(
    '#type' => 'checkboxes',
    '#title' => t('Supported Types'),
    '#options' => $options,
    '#default_value' => variable_get('emaudio_customurl_supported_types', array_keys($options)),
    '#description' => t('Select the video types you wish to support. When a custom url with that type is entered into an embedded video field, it will be parsed and displayed appropriately. If a type is not supported, then it will be ignored.'),
  );

  return $form;
}

/**
 * Retrieves an array of supported custom URL types.
 */
function emaudio_customurl_default_types() {
  return array(
    'mp3' => t('MPEG-1 Audio Layer 3 (MP3)'),
    'wav' => t('Waveform Audio Format (WAV)'),
    'ra' => t('Real Audio (RA)'),
    'mid' => t('Musical Instrument Digital Interface (MID)'),
  );
}

/**
 * Implementation of hook emaudio_PROVIDER_extract().
 *
 * This is called to extract the video code from a pasted URL or embed code.
 *
 * @param $embed
 *   An optional string with the pasted URL or embed code.
 * @return
 *   Either an array of regex expressions to be tested, or a string with the
 *   audio code to be used. If the hook tests the code itself, it should return
 *   either the string of the audio code (if matched), or an empty array.
 *   Otherwise, the calling function will handle testing the embed code against
 *   each regex string in the returned array.
 */
function emaudio_customurl_extract($embed = '') {
  $types = emaudio_customurl_default_types();
  $types = implode('|', array_keys($types));
  $baseurl = preg_quote(url(null, array('absolute' => TRUE)), '@');

  return array(
    '@' . $baseurl . '(.*' . '\.(?:' . $types . ')' . '(\?.*)?)@i',
    '@(.*\.(?:'. $types .')(\?.*)?)@i'
  );
}

/**
 * Implementation of hook_PROVIDER_data().
 */
function emaudio_customurl_data($field, $item) {
  $data = array();
  // adding the version control
  $data['emaudio_customurl_data_version'] = 1;

  // attempt to get info from headers
  $response = emfield_request_header('customurl', $item['embed']);

  if ($response->code == 200) {
    $data['url'] = $item['embed'];
    $data['size'] = $response->headers['Content-Length'];
    $data['mime'] = $response->headers['Content-Type'];
  }

  // @todo replace ['type'] with converted mime info if available
  $types = emaudio_customurl_default_types();
  $types = implode('|', array_keys($types));
  $regex = '@\.('. $types .')@i';
  if (preg_match($regex, $item['embed'], $matches)) {
    $data['type'] = $matches[1];
  }
  return $data;
}

/**
 * hook emfield_PROVIDER_rss
 */
function emaudio_customurl_rss($item, $teaser = NULL) {
  if ($item['value']) {
    if ($item['data']['emaudio_customurl_data_version'] >= 1) {
      $data = $item['data'];
    }
    else {
      $data = emaudio_customurl_data(NULL, $item);
    }

    $file = array();
    if ($data['size'] > 0) {
      $file['filepath'] = $data['url'];
      $file['filesize'] = $data['size'];
      $file['filemime'] = $data['mime'];
    }

    return $file;
  }
}

/**
 * hook emaudio_PROVIDER_audio
 * this actually displays the full/normal-sized video we want, usually on the default page view
 *  @param $embed
 *    the video code for the audio to embed
 *  @param $width
 *    the width to display the audio
 *  @param $height
 *    the height to display the audio
 *  @param $field
 *    the field info from the requesting node
 *  @param $item
 *    the actual content from the field
 *  @return
 *    the html of the embedded audio
 */
function emaudio_customurl_audio($url = NULL, $width = 0, $height = 0, $field = NULL, $data = array(), $node = NULL, $autoplay = FALSE) {
  // Construct the options.
  $options = array(
    'clip' => array(
      'url' => $url,
      'autoPlay' => $autoplay,
    ),
  );

  // Pass in the width and height of the element.
  $attributes = array(
    'style' => "width:{$width}px;height:{$height}px;",
  );

  // The ID should be the field's name.
  $id = isset($field['field_name']) ? $field['field_name'] : 'customurl';

  // Use Flowplayer to display the custom URL.
  return theme('flowplayer', $options, $id, $attributes);
}

